
##
## action of Aut(G) on T
##
AutGroupInducedAction := function( A )
    local G, T, t, a, m, M;
    G := A!.group;
    T := FittingSubgroup(G);
    t := Pcp(T, "snf");
    a := A!.induc;
    m := List(a, x -> List(t, y -> ExponentsByPcp(t, Image(x,y))));
    M := Group(m, IdentityMat(Length(t)));
    SetSize(M, A!.fsize);
    return M;
end;

##
## technical detail ...
##
ExponentsByDer := function( D, auto )
    local s;
    if D = fail then return []; fi;
    s := List(D.h, x -> ExponentsByPcp(D.t, x^-1*Image( auto, x )));
    return SolutionIntMat( D.mat, Concatenation(s) );
end;

##
## technical detail ...
##
DerivationInfo := function( der )
    local G, T, h, t, d;
    if Length(der) = 0 then return fail; fi;
    G := Source(der[1]);
    T := FittingSubgroup(G);
    h := Pcp(G,T);
    t := Pcp(T);
    d := List(der, y -> List(h, x -> ExponentsByPcp(t, x^-1*Image(y,x))));
    return rec( mat := List(d, Concatenation), h := h, t := t );
end;

IsPolycyclicAutGrpCG := function(A)
    local M;
    M := AutGroupInducedAction(A);
    return IsPolycyclicMatGroup(M);
end;

##
## IsomorphismPcpGroup
##
IsomorphismPcpGroupAutGrpCG := function(A)
    local M, H, K, D, iso, gen, inv, rel, der, new, m, i, j, k, h, C, c,
          r, a, b, v, P, f, w;

    # induce
    M := AutGroupInducedAction(A);
    if not IsPolycyclicMatGroup(M) then return fail; fi;

    # pcp pres for quotient
    iso := IsomorphismPcpGroup(M);
    H := Image(iso);

    # determine preimages and some general set up
    a := A!.induc;
    m := GeneratorsOfGroup(M);
    h := List(m, x -> Image(iso, x));
    gen := CgsParallel(h, a)[2];
    inv := List(gen, x -> x^-1);
    rel := RelativeOrdersOfPcp( Pcp(H) );
    der := A!.deriv;

    # set up for exponent computation
    D := DerivationInfo( der );
    if D = fail then
        return GroupHomomorphismByImagesNC( A, H, gen, Igs(H) );
    fi;

    # action on der
    m := List(gen, x -> NullMat(Length(der), Length(der)));
    for i in [1..Length(gen)] do
        for j in [1..Length(der)] do
            m[i][j] := ExponentsByDer( D, inv[i] * der[j] * gen[i] );
        od;
    od;
    C := CRRecordByMats( H, m );

    # extension with der
    c := [];
    for k in [1..Length(C.enumrels)] do
        i := C.enumrels[k][1];
        j := C.enumrels[k][2];
        r := C.relators[i][j];
        if j < i then
            a := inv[j] * gen[i] * gen[j];
        elif j > i then
            a := gen[j-i] * gen[i] * inv[j-i];
        else
            a := gen[i]^rel[i];
        fi;
        if Length(r) > 0 then
            b := Product( List( r, x -> gen[x[1]]^x[2] ) );
        else
            b := One(A);
        fi;
        v := ExponentsByDer( D, b^-1*a );
        Append(c, v);
    od;

    # create extension
    K := ExtensionCR(C, c);
    a := Concatenation(gen, der);

    # check
    if CHECK then
        P := PcpGroupToFpGroup(K);
        f := FreeGeneratorsOfFpGroup(P);
        r := RelatorsOfFpGroup(P);
        for i in [1..Length(r)] do
            w := MappedWord( r[i], f, a );
            if w <> One(A) then Error("wrong pres"); fi;
        od;
    fi;

    return GroupHomomorphismByImagesNC( A, K, a, Igs(K) );
end;

