
##
## autos inducing the identity of G/T and on T
##
DerivationAutosCG := function(G)
    local C, n, z, der, i, k, a;

    # get info
    AddExtensionInfoCG(G);
    C := G!.crrec;
    n := Length(C.f);

    # compute 1-cocycles
    z := OneCocyclesCR(C);

    # translate to autos
    der := [];
    for i in [1..Length(z)] do
        k := List( CutVector(z[i], n), x -> MappedVector(x,C.t));
        k := List([1..n], j -> C.f[j]*k[j]);
        k := Concatenation( k, C.t!.gens );
        a := GroupHomomorphismByImagesNC(G, G, C.h, k);
        SetIsBijective(a, true);
        Add(der, a);
    od;

    return der;
end;

##
## header function
##
AutGroupCrystPcpGroup := function( G )
    local der, com, ind, i, A, I;

    # automorphisms inducing the identity on G/T and on T
    der := DerivationAutosCG( G );

    # compatible pairs
    com := CompatiblePairsCG( G );

    # inducible pairs
    I := InduciblePairsCG( G, com );
    ind := ShallowCopy(GeneratorsOfGroup(I));

    # lift
    for i in [1..Length(ind)] do
        ind[i] := LiftAutoCG( G, ind[i] );
    od;

    A := Group(Concatenation( ind, der ));
    SetSize(A, infinity);
    A!.induc := ind;
    A!.deriv := der;
    A!.fsize := Size(I);
    A!.group := G;
    return A;
end;

